"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.withWeb3 = withWeb3;
exports.Web3Consumer = exports.default = void 0;

require("core-js/modules/web.dom.iterable");

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _web = _interopRequireDefault(require("web3"));

var w3w = _interopRequireWildcard(require("web3-webpacked"));

var _throttleDebounce = require("throttle-debounce");

var _defaultScreens = require("./defaultScreens");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

// define error names
const networkErrorName = w3w.networkErrorName;
const deniedAccessErrorName = 'EthereumAccountAccessDenied';

const Web3Context = _react.default.createContext();

class Web3Provider extends _react.Component {
  constructor(props) {
    super(props);
    this.state = {
      noWeb3: false,
      web3Error: null,
      initialized: false,
      w3w: {
        web3js: undefined,
        account: undefined,
        networkId: undefined
      }
    };
    this.pollId = undefined;
    this.web3Poll = this.web3Poll.bind(this);
  }

  componentDidMount() {
    this.initialize();
  }

  async initialize() {
    const _window = window,
          web3 = _window.web3,
          ethereum = _window.ethereum; // try to initialize web3js

    let web3js; // modern dapp browsers...

    if (ethereum) {
      // request account access...
      try {
        await ethereum.enable();
        web3js = new _web.default(ethereum);
      } catch (error) {
        const deniedAccessError = Error(error);
        deniedAccessError.name = deniedAccessErrorName;
        this.setError(deniedAccessError);
        return;
      } // legacy dapp browsers...

    } else if (web3 && web3.currentProvider) {
      web3js = new _web.default(web3.currentProvider); // no web3...
    } else {
      this.setState({
        noWeb3: true
      });
      return;
    } // initialize w3w utility functions and inject into state


    w3w._setEthereumVariableGetters({
      web3js: () => this.state.w3w.web3js,
      account: () => this.state.w3w.account,
      networkId: () => this.state.w3w.networkId
    });

    const initializeWeb3 = w3w.initializeWeb3,
          getWeb3js = w3w.getWeb3js,
          getAccount = w3w.getAccount,
          getNetworkId = w3w.getNetworkId,
          networkErrorName = w3w.networkErrorName,
          _setEthereumVariableGetters = w3w._setEthereumVariableGetters,
          w3wToInject = _objectWithoutProperties(w3w, ["initializeWeb3", "getWeb3js", "getAccount", "getNetworkId", "networkErrorName", "_setEthereumVariableGetters"]);

    this.setState(oldState => {
      return {
        w3w: _objectSpread({}, oldState.w3w, {
          web3js: web3js
        }, w3wToInject)
      };
    }, () => this.web3Poll(true));
  }

  web3Poll(first) {
    // fetch current network
    const networkPromise = new Promise((resolve, reject) => {
      this.state.w3w.web3js.eth.net.getId().then(networkId => {
        if (!this.props.supportedNetworks.includes(networkId)) {
          const error = Error(`Current network id '${networkId}' is unsupported.`);
          error.name = networkErrorName;
          return reject(error);
        }

        if (networkId !== this.state.w3w.networkId) {
          this.setState(oldState => {
            return {
              w3w: _objectSpread({}, oldState.w3w, {
                networkId: networkId
              })
            };
          }, resolve);
        } else {
          resolve(false);
        }
      }).catch(error => {
        reject(error);
      });
    }); // check for default account changes

    const accountPromise = new Promise((resolve, reject) => {
      this.state.w3w.web3js.eth.getAccounts().then(accounts => {
        const account = accounts === undefined || accounts[0] === undefined ? null : accounts[0];

        if (account !== this.state.w3w.account) {
          this.setState(oldState => {
            return {
              w3w: _objectSpread({}, oldState.w3w, {
                account: account
              })
            };
          }, resolve);
        } else {
          resolve(false);
        }
      }).catch(error => {
        reject(error);
      });
    });
    Promise.all([networkPromise, accountPromise]).then(() => {
      if (first) {
        this.pollId = setInterval((0, _throttleDebounce.throttle)(this.props.pollTime - 1, this.web3Poll), this.props.pollTime);
        this.setState({
          initialized: true
        });
      }

      if (this.state.web3Error !== null) {
        this.setError(null);
      }
    }).catch(error => {
      this.setError(error);
    });
  }

  setError(error) {
    this.setState({
      web3Error: error
    });
  }

  clearPoll() {
    if (this.pollId !== undefined) {
      clearTimeout(this.pollId);
      this.pollId = undefined;
    }
  }

  getScreen(name) {
    return this.props.screens[name] || defaultScreens[name];
  }

  componentWillUnmount() {
    this.clearPoll();
  }

  render() {
    const children = this.props.children;

    if (this.state.noWeb3) {
      const Screen = this.getScreen('noWeb3');
      return _react.default.createElement(Screen, null);
    }

    if (this.state.web3Error !== null) {
      if (this.state.web3Error.name === networkErrorName) {
        const Screen = this.getScreen('unsupportedNetwork');
        const supportedNetworkNames = this.props.supportedNetworks.map(id => w3w.getNetworkName(id));
        return _react.default.createElement(Screen, {
          supportedNetworkNames: supportedNetworkNames
        });
      } else if (this.state.web3Error.name === deniedAccessErrorName) {
        const Screen = this.getScreen('permissionNeeded');
        return _react.default.createElement(Screen, null);
      } else {
        const Screen = this.getScreen('web3Error');
        return _react.default.createElement(Screen, {
          error: this.state.web3Error
        });
      }
    }

    if (!this.state.initialized) {
      const Screen = this.getScreen('initializing');
      return _react.default.createElement(Screen, null);
    }

    if (this.state.w3w.account === null) {
      const Screen = this.getScreen('unlockNeeded');
      return _react.default.createElement(Screen, null);
    }

    return _react.default.createElement(Web3Context.Provider, {
      value: this.state.w3w
    }, children);
  }

}

Web3Provider.propTypes = {
  screens: _propTypes.default.shape({
    initializing: _propTypes.default.any,
    noWeb3: _propTypes.default.any,
    permissionNeeded: _propTypes.default.any,
    unlockNeeded: _propTypes.default.any,
    unsupportedNetwork: _propTypes.default.any,
    web3Error: _propTypes.default.any
  }),
  pollTime: _propTypes.default.number,
  supportedNetworks: _propTypes.default.arrayOf(_propTypes.default.number),
  children: _propTypes.default.node
};
const defaultScreens = {
  initializing: _defaultScreens.Initializing,
  noWeb3: _defaultScreens.NoWeb3,
  permissionNeeded: _defaultScreens.PermissionNeeded,
  unlockNeeded: _defaultScreens.UnlockNeeded,
  unsupportedNetwork: _defaultScreens.UnsupportedNetwork,
  web3Error: _defaultScreens.Web3Error
};
Web3Provider.defaultProps = {
  screens: defaultScreens,
  pollTime: 1000,
  supportedNetworks: [1, 3, 4, 42],
  children: ''
};
var _default = Web3Provider; // withWeb3 is an HOC wrapper that gives components access to the w3w context

exports.default = _default;

function withWeb3(ComponentToWrap) {
  class WithWeb3 extends _react.Component {
    render() {
      return _react.default.createElement(Web3Context.Consumer, null, context => _react.default.createElement(ComponentToWrap, _extends({}, this.props, {
        w3w: context
      })));
    }

  }

  WithWeb3.displayName = `WithWeb3(${ComponentToWrap.displayName || ComponentToWrap.name || 'Component'})`;
  return WithWeb3;
} // alternatively, the consumer is exposed to allow direct use of render props


class Web3Consumer extends _react.Component {
  render() {
    const _this$props = this.props,
          recreateOnNetworkChange = _this$props.recreateOnNetworkChange,
          recreateOnAccountChange = _this$props.recreateOnAccountChange,
          children = _this$props.children;
    return _react.default.createElement(Web3Context.Consumer, null, context => _react.default.createElement(_react.Fragment, {
      key: recreateOnNetworkChange ? context.networkId : undefined
    }, _react.default.createElement(_react.Fragment, {
      key: recreateOnAccountChange ? context.account : undefined
    }, children(context))));
  }

}

exports.Web3Consumer = Web3Consumer;
Web3Consumer.propTypes = {
  recreateOnNetworkChange: _propTypes.default.bool,
  recreateOnAccountChange: _propTypes.default.bool,
  children: _propTypes.default.func.isRequired
};
Web3Consumer.defaultProps = {
  recreateOnAccountChange: true,
  recreateOnNetworkChange: true
};